@@ -15,12 +15,8 @@ package org.tmatesoft.svn.core.internal.io.dav;
 import java.util.Iterator;
 import java.util.Map;
 
-import org.tmatesoft.svn.core.SVNErrorCode;
-import org.tmatesoft.svn.core.SVNErrorMessage;
-import org.tmatesoft.svn.core.SVNException;
-import org.tmatesoft.svn.core.SVNProperties;
-import org.tmatesoft.svn.core.SVNProperty;
-import org.tmatesoft.svn.core.SVNPropertyValue;
+import org.tmatesoft.svn.core.*;
+import org.tmatesoft.svn.core.internal.io.dav.handlers.DAVOptionsHandler;
 import org.tmatesoft.svn.core.internal.io.dav.handlers.DAVPropertiesHandler;
 import org.tmatesoft.svn.core.internal.io.dav.http.HTTPHeader;
 import org.tmatesoft.svn.core.internal.io.dav.http.HTTPStatus;
@@ -29,6 +25,7 @@ import org.tmatesoft.svn.core.internal.util.SVNHashMap;
 import org.tmatesoft.svn.core.internal.util.SVNPathUtil;
 import org.tmatesoft.svn.core.internal.wc.SVNErrorManager;
 import org.tmatesoft.svn.core.io.ISVNEditor;
+import org.tmatesoft.svn.core.wc.SVNRevision;
 import org.tmatesoft.svn.util.SVNLogType;
 
 /**
@@ -178,12 +175,31 @@ public class DAVUtil {
         return vcc.getString();
     }
 
+    public static DAVBaselineInfo getStableURL(DAVConnection connection, DAVRepository repos, String path, long revision,
+                                                  boolean includeType, boolean includeRevision, DAVBaselineInfo info) throws SVNException {
+        if (connection.hasHttpV2Support()) {
+            info = info == null ? new DAVBaselineInfo() : info;
+            if (SVNRevision.isValidRevisionNumber(revision)) {
+                info.revision = revision;
+            } else {
+                info.revision = getLatestRevisionHttpV2(connection);
+            }
+            info.baselineBase = SVNPathUtil.append(connection.myRevRootStub, String.valueOf(info.revision));
+            info.baselinePath = connection.getRelativePath(path);
+            return info;
+        } else {
+            return getBaselineInfo(connection, repos, path, revision, includeType, includeRevision, info);
+        }
+    }
+
     public static DAVBaselineInfo getBaselineInfo(DAVConnection connection, DAVRepository repos, String path, long revision,
                                                   boolean includeType, boolean includeRevision, DAVBaselineInfo info) throws SVNException {
+        info = info == null ? new DAVBaselineInfo() : info;
+
         DAVElement[] properties = includeRevision ? DAVElement.BASELINE_PROPERTIES : new DAVElement[] {DAVElement.BASELINE_COLLECTION};
         DAVProperties baselineProperties = getBaselineProperties(connection, repos, path, revision, properties);
 
-        info = info == null ? new DAVBaselineInfo() : info;
+
         info.baselinePath = baselineProperties.getURL();
         SVNPropertyValue baseValue = baselineProperties.getPropertyValue(DAVElement.BASELINE_COLLECTION);
         info.baselineBase = baseValue == null ? null : baseValue.getString();
@@ -251,6 +267,21 @@ public class DAVUtil {
         return properties;
     }
 
+    public static long getLatestRevisionHttpV2(DAVConnection davConnection) throws SVNException {
+        HTTPStatus status = davConnection.doOptions(davConnection.getLocation().getPath());
+        if (status.getCode() != 200) {
+            SVNErrorMessage err = SVNErrorMessage.create(SVNErrorCode.RA_DAV_OPTIONS_REQ_FAILED,
+                    "OPTIONS request (for latest revision) got HTTP response code {0}",
+                    new Integer(status.getCode()));
+            SVNErrorManager.error(err, SVNLogType.NETWORK);
+        }
+        if (!SVNRevision.isValidRevisionNumber(davConnection.myLatestRevision)) {
+            SVNErrorMessage errorMessage = SVNErrorMessage.create(SVNErrorCode.RA_DAV_OPTIONS_REQ_FAILED, "The OPTIONS response did not include the youngest revision");
+            SVNErrorManager.error(errorMessage, SVNLogType.NETWORK);
+        }
+        return davConnection.myLatestRevision;
+    }
+
     public static SVNProperties filterProperties(DAVProperties source, SVNProperties target) {
         target = target == null ? new SVNProperties() : target;
         for (Iterator props = source.getProperties().entrySet().iterator(); props.hasNext();) {
@@ -285,8 +316,15 @@ public class DAVUtil {
         return null;
     }
 
+    /**
+     * @deprecated for binary compatibility only
+     */
     public static void setSpecialWCProperties(SVNProperties props, DAVElement property, SVNPropertyValue propValue) {
-        String propName = convertDAVElementToPropName(property);
+        setSpecialWCProperties(props, property, propValue, false);
+    }
+
+    public static void setSpecialWCProperties(SVNProperties props, DAVElement property, SVNPropertyValue propValue, boolean isDir) {
+        String propName = convertDAVElementToPropName(property, isDir);
         if (propName != null) {
             props.put(propName, propValue);
         }
@@ -294,7 +332,7 @@ public class DAVUtil {
 
     public static void setSpecialWCProperties(ISVNEditor editor, boolean isDir, String path, DAVElement property, 
             SVNPropertyValue propValue) throws SVNException {
-        String propName = convertDAVElementToPropName(property);
+        String propName = convertDAVElementToPropName(property, isDir);
         if (propName != null) {
             if (isDir) {
                 editor.changeDirProperty(propName, propValue);
@@ -321,7 +359,7 @@ public class DAVUtil {
         return "/";
     }
 
-    private static String convertDAVElementToPropName(DAVElement property) {
+    private static String convertDAVElementToPropName(DAVElement property, boolean isDir) {
         String propName = null;
         if (property == DAVElement.VERSION_NAME) {
             propName = SVNProperty.COMMITTED_REVISION;    
@@ -331,11 +369,68 @@ public class DAVUtil {
             propName = SVNProperty.COMMITTED_DATE;
         } else if (property == DAVElement.REPOSITORY_UUID) {
             propName = SVNProperty.UUID;
-        } else if (property == DAVElement.MD5_CHECKSUM) {
+        } else if (property == DAVElement.MD5_CHECKSUM && !isDir) {
             propName = SVNProperty.CHECKSUM;
-        } else if (property == DAVElement.SHA1_CHECKSUM) {
+        } else if (property == DAVElement.SHA1_CHECKSUM && !isDir) {
             propName = SVNProperty.SVNKIT_SHA1_CHECKSUM;
         }
         return propName;
     }
+
+    public static SVNErrorMessage createUnexpectedStatusErrorMessage(HTTPStatus httpStatus, String method, String path) {
+        final int code = httpStatus.getCode();
+        final String location = httpStatus.getHeader().getFirstHeaderValue(HTTPHeader.LOCATION_HEADER);
+        if (code != 405) {
+            SVNErrorMessage errorMessage = createDefaultUnexpectedStatusErrorMessage(httpStatus, path, location);
+            if (errorMessage != null) {
+                return errorMessage;
+            }
+        }
+        switch (code) {
+            case 201:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, "Path '{{0}}' unexpectedly created", path);
+            case 204:
+                return SVNErrorMessage.create(SVNErrorCode.FS_ALREADY_EXISTS, "Path '{{0}}' already exists", path);
+            case 405:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_METHOD_NOT_ALLOWED, "The HTTP method '{{0}}' is not allowed on '{{0}}'", method, path);
+            default:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, "Unexpected HTTP status {{0}} '{{1}}' on '{{2}}' request to '{{3}}'", code, httpStatus.getReason(), method, path);
+        }
+    }
+
+    private static SVNErrorMessage createDefaultUnexpectedStatusErrorMessage(HTTPStatus httpStatus, String path, String location) {
+        final int code = httpStatus.getCode();
+        switch (code) {
+            case 301:
+            case 302:
+            case 303:
+            case 307:
+            case 308:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_RELOCATED, (code == 301) ? "Repository moved permanently to '{{0}}'" : "Repository moved temporarily to '%s'", location);
+            case 403:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_FORBIDDEN, "Access to '{{0}}' forbidden", path);
+            case 404:
+                return SVNErrorMessage.create(SVNErrorCode.FS_NOT_FOUND, "'{{0}}' path not found", path);
+            case 405:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_METHOD_NOT_ALLOWED, "HTTP method is not allowed on '{{0}}'", path);
+            case 409:
+                return SVNErrorMessage.create(SVNErrorCode.FS_CONFLICT, "'{{0}}' conflicts", path);
+            case 412:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_PRECONDITION_FAILED, "Precondition on '{{0}}' failed", path);
+            case 423:
+                return SVNErrorMessage.create(SVNErrorCode.FS_NO_LOCK_TOKEN, "'{{0}}': no lock token available", path);
+            case 411:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, "DAV request failed: 411 Content length required. The server or an intermediate proxy does not accept " +
+                        "chunked encoding. Try setting 'http-chunked-requests' to 'auto' or 'no' in your client configuration.");
+            case 500:
+                return SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, "Unexpected server error {{0}} '{{1}}' on '{{2}}'", code, httpStatus.getReason(), path);
+            case 501:
+                return SVNErrorMessage.create(SVNErrorCode.UNSUPPORTED_FEATURE, "The requested feature is not supported by '%s'", path);
+        }
+
+        if (code >= 300 || code <= 199) {
+            return SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, "Unexpected HTTP status {{0}} '{{1}}' on '{{2}}'", code, httpStatus.getReason(), path);
+        }
+        return null;
+    }
 }
