@@ -11,6 +11,7 @@
  */
 package org.tmatesoft.svn.core.internal.io.dav.http;
 
+import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
@@ -97,47 +98,31 @@ public final class HTTPSSLKeyManager implements X509KeyManager {
         return result;
     }
 
+    /**
+     * @deprecated
+     * 
+     * @param clientCertFile
+     * @param clientCertPassword
+     * @return
+     * @throws SVNException
+     */
     public static KeyManager[] loadClientCertificate(File clientCertFile, String clientCertPassword) throws SVNException {
-        char[] passphrase = null;
-        if (clientCertPassword == null || clientCertPassword.length() == 0) {
+        return loadClientCertificate(clientCertFile, clientCertPassword != null ? clientCertPassword.toCharArray() : null);
+    }
+
+    public static KeyManager[] loadClientCertificate(byte[] clientCert, char[] clientCertPassword) throws SVNException {
+        if (clientCertPassword == null || clientCertPassword.length == 0) {
             // Client certificates without an passphrase can't be received from Java Keystores. 
             throw new SVNException(SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, "No client certificate passphrase supplied (did you forget to specify?).\n" +
                     "Note that client certificates with empty passphrases can''t be used. In this case please re-create the certificate with a passphrase."));
         }
-        if (clientCertPassword != null) {
-            passphrase = clientCertPassword.toCharArray();
-        }
         KeyManager[] result = null;
         KeyStore keyStore = null;
-        if (clientCertFile != null && clientCertFile.getName().startsWith(SVNSSLAuthentication.MSCAPI)) {
-            SVNDebugLog.getDefaultLog().logError(SVNLogType.CLIENT,"using mscapi");
-            try {
-                keyStore = KeyStore.getInstance("Windows-MY");
-                SVNDebugLog.getDefaultLog().logFine(SVNLogType.NETWORK, "using my windows store");
-                if (keyStore != null) {
-                    keyStore.load(null, null);
-                }
-                KeyManagerFactory kmf = null;
-                if (keyStore != null) {
-                    kmf = KeyManagerFactory.getInstance("SunX509");
-                    if (kmf != null) {
-                        kmf.init(keyStore, passphrase);
-                        result = kmf.getKeyManagers();
-                    }
-                }
-                return result;
-            }
-            catch (Throwable th) {
-                SVNDebugLog.getDefaultLog().logFine(SVNLogType.NETWORK, th);
-                throw new SVNException(SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, "loadClientCertificate ms capi with file - should not be called: "+th.getMessage(), null, SVNErrorMessage.TYPE_ERROR, th), th);
-            }
-
-        }
-        final InputStream is = SVNFileUtil.openFileForReading(clientCertFile, SVNLogType.NETWORK);
+        final InputStream is = new ByteArrayInputStream(clientCert);
         try {
             keyStore = KeyStore.getInstance("PKCS12");
             if (keyStore != null) {
-                keyStore.load(is, passphrase);
+                keyStore.load(is, clientCertPassword);
             }
         }
         catch (Throwable th) {
@@ -153,7 +138,72 @@ public final class HTTPSSLKeyManager implements X509KeyManager {
             try {
                 kmf = KeyManagerFactory.getInstance("SunX509");
                 if (kmf != null) {
-                    kmf.init(keyStore, passphrase);
+                    kmf.init(keyStore, clientCertPassword);
+                    result = kmf.getKeyManagers();
+                }
+            }
+            catch (Throwable th) {
+                SVNDebugLog.getDefaultLog().logFine(SVNLogType.NETWORK, th);
+                throw new SVNException(SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, th.getMessage()), th);
+            }
+        }
+
+        return result;
+    }
+
+    public static KeyManager[] loadClientCertificate(File clientCertFile, char[] clientCertPassword) throws SVNException {
+        if (clientCertPassword == null || clientCertPassword.length == 0) {
+            // Client certificates without an passphrase can't be received from Java Keystores. 
+            throw new SVNException(SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, "No client certificate passphrase supplied (did you forget to specify?).\n" +
+                    "Note that client certificates with empty passphrases can''t be used. In this case please re-create the certificate with a passphrase."));
+        }
+        KeyManager[] result = null;
+        KeyStore keyStore = null;
+        if (clientCertFile != null && clientCertFile.getName().startsWith(SVNSSLAuthentication.MSCAPI)) {
+            SVNDebugLog.getDefaultLog().logError(SVNLogType.CLIENT,"using mscapi");
+            try {
+                keyStore = KeyStore.getInstance("Windows-MY");
+                SVNDebugLog.getDefaultLog().logFine(SVNLogType.NETWORK, "using my windows store");
+                if (keyStore != null) {
+                    keyStore.load(null, null);
+                }
+                KeyManagerFactory kmf = null;
+                if (keyStore != null) {
+                    kmf = KeyManagerFactory.getInstance("SunX509");
+                    if (kmf != null) {
+                        kmf.init(keyStore, clientCertPassword);
+                        result = kmf.getKeyManagers();
+                    }
+                }
+                return result;
+            }
+            catch (Throwable th) {
+                SVNDebugLog.getDefaultLog().logFine(SVNLogType.NETWORK, th);
+                throw new SVNException(SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, "loadClientCertificate ms capi with file - should not be called: "+th.getMessage(), null, SVNErrorMessage.TYPE_ERROR, th), th);
+            }
+
+        }
+        final InputStream is = SVNFileUtil.openFileForReading(clientCertFile, SVNLogType.NETWORK);
+        try {
+            keyStore = KeyStore.getInstance("PKCS12");
+            if (keyStore != null) {
+                keyStore.load(is, clientCertPassword);
+            }
+        }
+        catch (Throwable th) {
+            SVNDebugLog.getDefaultLog().logFine(SVNLogType.NETWORK, th);
+            throw new SVNException(SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, th.getMessage(), null, SVNErrorMessage.TYPE_ERROR, th), th);
+        }
+        finally {
+            SVNFileUtil.closeFile(is);
+        }
+        KeyManagerFactory kmf = null;
+
+        if (keyStore != null) {
+            try {
+                kmf = KeyManagerFactory.getInstance("SunX509");
+                if (kmf != null) {
+                    kmf.init(keyStore, clientCertPassword);
                     result = kmf.getKeyManagers();
                 }
             }
@@ -167,11 +217,12 @@ public final class HTTPSSLKeyManager implements X509KeyManager {
     }
 
     public KeyManager[] loadClientCertificate(SVNSSLAuthentication sslAuthentication) throws SVNException {
-        String clientCertPassword = sslAuthentication.getPassword();
+        char[] clientCertPassword = sslAuthentication.getPasswordValue();
         String clientCertPath = sslAuthentication.getCertificatePath();
         File clientCertFile = sslAuthentication.getCertificateFile();
+        byte[] clientCertData = sslAuthentication.getCertificate();
         
-        char[] passphrase = clientCertPassword == null || clientCertPassword.length() == 0 ? new char[0] : clientCertPassword.toCharArray(); 
+        char[] passphrase = clientCertPassword == null ? new char[0] : clientCertPassword; 
         String realm = clientCertPath;
         SVNAuthentication auth = null;
 
@@ -203,7 +254,8 @@ public final class HTTPSSLKeyManager implements X509KeyManager {
 
         while(true) {
             try {
-                final InputStream is = SVNFileUtil.openFileForReading(clientCertFile, SVNLogType.NETWORK);
+                final InputStream is = clientCertData != null ? 
+                        new ByteArrayInputStream(clientCertData) : SVNFileUtil.openFileForReading(clientCertFile, SVNLogType.NETWORK);
                 try {
                     keyStore = KeyStore.getInstance("PKCS12");
                     if (keyStore != null) {
@@ -216,7 +268,7 @@ public final class HTTPSSLKeyManager implements X509KeyManager {
                     BasicAuthenticationManager.acknowledgeAuthentication(true, ISVNAuthenticationManager.SSL, realm, null, auth, url, authenticationManager);
                 } else {
                     BasicAuthenticationManager.acknowledgeAuthentication(true, ISVNAuthenticationManager.SSL, clientCertPath, null,
-                            new SVNPasswordAuthentication("", clientCertPassword, sslAuthentication.isStorageAllowed(), sslAuthentication.getURL(), false), url, authenticationManager);
+                            SVNPasswordAuthentication.newInstance("", clientCertPassword, sslAuthentication.isStorageAllowed(), sslAuthentication.getURL(), false), url, authenticationManager);
                 }
                 break;
             } catch (IOException io) {
@@ -228,7 +280,7 @@ public final class HTTPSSLKeyManager implements X509KeyManager {
                     auth = authenticationManager.getFirstAuthentication(ISVNAuthenticationManager.SSL, realm, sslAuthentication.getURL());
                 }
                 if (auth instanceof SVNPasswordAuthentication) {
-                    passphrase = ((SVNPasswordAuthentication) auth).getPassword().toCharArray();
+                    passphrase = ((SVNPasswordAuthentication) auth).getPasswordValue();
                 } else {
                     auth = null;
                     SVNErrorManager.cancel("authentication cancelled", SVNLogType.NETWORK);
@@ -437,7 +489,11 @@ public final class HTTPSSLKeyManager implements X509KeyManager {
                             ((ISVNSSLPasspharsePromptSupport) authenticationManager).isSSLPassphrasePromtSupported()) {
                         keyManagers = loadClientCertificate(myAuthentication);
                     } else {
-                        keyManagers = loadClientCertificate(myAuthentication.getCertificateFile(), myAuthentication.getPassword());
+                        if (myAuthentication.getCertificate() != null) {
+                            keyManagers = loadClientCertificate(myAuthentication.getCertificate(), myAuthentication.getPasswordValue());
+                        } else {
+                            keyManagers = loadClientCertificate(myAuthentication.getCertificateFile(), myAuthentication.getPasswordValue());
+                        }
                     }
                 }
                 
