@@ -33,14 +33,11 @@ import org.tmatesoft.svn.core.SVNProperties;
 import org.tmatesoft.svn.core.SVNProperty;
 import org.tmatesoft.svn.core.SVNPropertyValue;
 import org.tmatesoft.svn.core.SVNRevisionProperty;
-import org.tmatesoft.svn.core.internal.db.SVNSqlJetStatement;
 import org.tmatesoft.svn.core.internal.util.SVNDate;
 import org.tmatesoft.svn.core.internal.util.SVNHashSet;
 import org.tmatesoft.svn.core.internal.util.SVNPathUtil;
-import org.tmatesoft.svn.core.internal.util.SVNSkel;
 import org.tmatesoft.svn.core.internal.wc.SVNErrorManager;
 import org.tmatesoft.svn.core.internal.wc.SVNFileUtil;
-import org.tmatesoft.svn.core.internal.wc17.db.statement.SVNWCDbStatements;
 import org.tmatesoft.svn.core.io.ISVNLockHandler;
 import org.tmatesoft.svn.core.io.SVNRepository;
 import org.tmatesoft.svn.util.SVNDebugLog;
@@ -54,14 +51,14 @@ import org.tmatesoft.svn.util.SVNLogType;
 public class FSCommitter {
 
     private static volatile boolean ourAutoUnlock;
-    
+
     private FSFS myFSFS;
     private FSTransactionRoot myTxnRoot;
     private FSTransactionInfo myTxn;
     private Collection<String> myLockTokens;
     private Map<String, String> myAutoUnlockPaths;
     private String myAuthor;
-    
+
     public static synchronized void setAutoUnlock(boolean autoUnlock) {
         ourAutoUnlock = autoUnlock;
     }
@@ -78,7 +75,7 @@ public class FSCommitter {
         myAutoUnlockPaths = new HashMap<String, String>();
         myAuthor = author;
     }
-    
+
     public Map<String, String> getAutoUnlockPaths() {
         return myAutoUnlockPaths;
     }
@@ -292,6 +289,13 @@ public class FSCommitter {
             FSHooks.runPreCommitHook(myFSFS.getRepositoryRoot(), myTxn.getTxnId());
         }
 
+        final SVNProperties txnProperties = myFSFS.getTransactionProperties(myTxn.getTxnId());
+        for (String propertyName : new HashSet<String>(txnProperties.nameSet())) {
+            if (propertyName.startsWith(SVNRevisionProperty.SVN_TXN_PREFIX)) {
+                myFSFS.setTransactionProperty(myTxn.getTxnId(), propertyName, null);
+            }
+        }
+
         long newRevision = SVNRepository.INVALID_REVISION;
 
         while (true) {
@@ -521,7 +525,7 @@ public class FSCommitter {
         }
 
         String commitTime = SVNDate.formatDate(new Date(System.currentTimeMillis()));
-        SVNProperties presetRevisionProperties = myFSFS.getTransactionProperties(myTxn.getTxnId()); 
+        SVNProperties presetRevisionProperties = myFSFS.getTransactionProperties(myTxn.getTxnId());
         if (presetRevisionProperties == null || !presetRevisionProperties.containsName(SVNRevisionProperty.DATE)) {
             myFSFS.setTransactionProperty(myTxn.getTxnId(), SVNRevisionProperty.DATE, SVNPropertyValue.create(commitTime));
         }
@@ -717,7 +721,7 @@ public class FSCommitter {
         final long sourcePredecessorCount = sourceNode.getCount();
         targetNode.setPredecessorId(sourceId);
         targetNode.setCount(sourcePredecessorCount != -1 ? sourcePredecessorCount + 1 : sourcePredecessorCount);
-        
+
         targetNode.setIsFreshTxnRoot(false);
         owner.putTxnRevisionNode(targetId, targetNode);
     }
@@ -758,7 +762,7 @@ public class FSCommitter {
     }
 
     public void allowLockedOperation(FSFS fsfs, String path, final String username, final Collection<String> lockTokens, boolean recursive, boolean haveWriteLock) throws SVNException {
-        
+
         path = SVNPathUtil.canonicalizeAbsolutePath(path);
         if (recursive) {
             ISVNLockHandler handler = new ISVNLockHandler() {
@@ -784,7 +788,7 @@ public class FSCommitter {
             }
         }
     }
-    
+
     private void scheduleForAutoUnlock(final String username, String path, SVNLock lock) {
         myAutoUnlockPaths.put(path, lock.getID());
     }
